@@ -3,7 +3,7 @@ import numpy as np
 from PlanetProfile.Utilities.defineStructs import ColorStruct, StyleStruct, \
     FigLblStruct, FigSizeStruct, FigMiscStruct
 
-configPlotsVersion = 1  # Integer number for config file version. Increment when new settings are added to the default config file.
+configPlotsVersion = 4  # Integer number for config file version. Increment when new settings are added to the default config file.
 Color = ColorStruct()
 Style = StyleStruct()
 FigLbl = FigLblStruct()
@@ -13,6 +13,9 @@ FigMisc = FigMiscStruct()
 """ Figure color options """
 Color.Induction = {'synodic': 'blue', 'orbital': 'purple', 'true anomaly': 'green', 'synodic harmonic': 'goldenrod'}  # Colors for inductOgram plots
 Color.ref = 'gray'
+Color.geotherm = 'white'
+Color.BdipInset = 'black'  # Color for inset box of surface induced dipole strength plots
+Color.innerCmapName = 'inferno'  # For plotting temperature profiles atop Perple_X data
 
 # Wedge diagram color options
 Color.none = '#FFFFFF00'
@@ -47,7 +50,6 @@ Color.silConvBot = 0.9
 Color.silConvN = 10
 Color.FeS = 'xkcd:puke'
 Color.Fe = '#2d3639'
-Color.innerCmapName = 'inferno'  # For plotting temperature profiles atop Perple_X data
 # Alternative color options for silicates
 Color.PALE_SILICATES = False  # Whether to use a lighter color scheme for silicate layers, or a more "orangey" saturated one
 Color.paleSilPorousCmapName = 'terrain'
@@ -61,11 +63,12 @@ Color.paleSilCondBot = 0.78
 Color.cmapName = {
     'none': 'copper',
     'PureH2O': 'cividis',
-    'Seawater': 'cool_r',
-    'MgSO4': 'winter_r',
+    'Seawater': 'cool',
+    'MgSO4': 'winter',
     'NH3': 'spring',
     'NaCl': 'summer',
-    'Ice': 'coolwarm_r'
+    'Ice': 'coolwarm_r',
+    'default': 'plasma'
 }
 # Select only a subset of the available colormap, if we choose to
 Color.cmapBounds = {
@@ -75,10 +78,11 @@ Color.cmapBounds = {
     'MgSO4': [0.0, 1.0],
     'NH3': [0.0, 1.0],
     'NaCl': [0.0, 1.0],
-    'Ice': [0.2, 0.8]
+    'Ice': [0.2, 0.8],
+    'default': [0.0, 1.0]
 }
 # Set temperature bounds to use for colormap normalization
-Color.Tbounds_K = [245.0, 300.0]
+Color.Tbounds_K = [255.0, 273.0]
 
 # Set upper bounds for max concentrations to use for darkening
 Color.saturation = {
@@ -98,15 +102,17 @@ Color.SetCmaps()
 
 
 """ Figure style options """
-Style.LS_dft = '-'  # Default line style to use on plots
-Style.LS_Sw = '-'  # Linestyle for Seawater
-Style.LS_Mg = '--'  # Linestyle for MgSO4
-Style.LS_sp = ':'  # Linestyle for special consideration models
-Style.LW_sal = 3  # Linewidth for higher salinity
-Style.LW_dil = 1  # Linewidth for dilute salinity
-Style.LW_std = 2  # Linewidth for standard salinity
-Style.LW_sound = 1.5  # LineWidth for sound speed plots
-Style.LW_seism = 1  # LineWidth for seismic plots (Attenuation)
+Style.GRIDS = False  # Whether to plot grids
+Style.LS = {'none': None, 'PureH2O': ':', 'Seawater': '-', 'MgSO4': '--', 'NH3': '-.', 'NaCl': '-'}  # LineStyle options for hydrosphere plots
+Style.LWlims = [0.5, 2]  # Bounds of linewidths to use for salinity mapping
+Style.MW_hydro = 3  # Marker size for hydrosphere plot endpoint, in the factor by which to multiply the linewidth.
+Style.MS_hydro = 'o'  # Marker style for hydrosphere plot endpoint
+Style.LW_std = 1.5  # Standard linewidth to use when not mapping as above
+Style.LW_sound = 1  # Linewidth for sound speeds on hydrosphere plot
+Style.LW_geotherm = 1  # Linewidth for geotherm on PT plots
+Style.LS_geotherm = '-'  # Linestyle for geotherm on PT plots
+Style.LW_seis = 1  # Linewidth for seismic plots
+Style.LS_seis = {'KS': '-', 'GS': '--', 'VP': '-', 'VS': '--', 'QS': '-', 'P': '-', 'T': '--', 'rho': '-.'}
 Style.LS_ref = {'none': None, 'PureH2O': '-', 'Seawater': ':', 'MgSO4': '--', 'NH3': '--', 'NaCl': '--'}  # Style for reference profiles
 Style.LW_ref = 0.75  # Linewidth for reference profiles
 Style.LS_Induction = {'synodic': '-', 'orbital': ':', 'true anomaly': ':', 'synodic harmonic': '--'}  # Style for inductOgram plots
@@ -114,39 +120,53 @@ Style.LW_Induction = {'synodic': 1.5, 'orbital': 1.5, 'true anomaly': 1.5, 'syno
 Style.MW_Induction = 2  # Marker size to use for induction scatter plots
 Style.MS_Induction = 'o'  # Marker style for induction scatter plots
 
+# Wedge diagrams
 Style.wedgeAngle_deg = 25  # Angular size of wedge diagrams in degrees
 Style.LW_wedge = 0.125  # Linewidth in pt for minor boundaries in wedge diagrams
 Style.LW_wedgeMajor = 0.375  # Linewidth in pt for major layer boundaries in wedge diagrams
 
+# Complex dipole plots
+Style.MW_dip = {'synodic': 5, 'orbital': 6, 'true anomaly': 6, 'synodic harmonic': 4.5}  # Marker size for each period in complex dipole plots
+Style.MS_dip = {'synodic': '*', 'orbital': 'o', 'true anomaly': 'P', 'synodic harmonic': 'h'}  # Marker style for each period in complex dipole plots
+Style.MAlims = [0, 1]  # Alpha channel (opacity) limits for markers
+Style.LS_BdipInset = '-'  # Linestyle for inset box 
+Style.LW_BdipInset = 0.5  # Linewidth for inset box
+
 
 """ Figure labels """
 # Note: Specific labels are set in PlanetProfile.Utilities.defineStructs.
 FigLbl.NEGATIVE_UNIT_POWERS = True  # Whether to use negative powers for units in latex tables, or instead a backslash.
+FigLbl.BODYNAME_IN_LABEL = True  # Whether to include the bodyname in legend labels for comparison plots including multiple bodies
 FigLbl.NAN_FOR_EMPTY = False  # Whether to use nan (or -) for empty layer parameters that were not calculated or not present.
+FigLbl.PFULL_IN_GPa = True  # Whether to plot P in GPa (or MPa) for full-body plots
+FigLbl.PHYDRO_IN_bar = False  # Whether to print P in bar (or MPa) for hydrosphere plots
 FigLbl.w_IN_WTPCT = False  # Whether to print salinities in wt% (or g/kg) in tables
-FigLbl.x_IN_WTPCT = True  # Whether to print silicate/core mass fractions in wt% (or g/kg) in tables
+FigLbl.T_IN_C = False  # Whether to print T in °C (or K) in plots
+FigLbl.x_IN_MOLPCT = True  # Whether to print silicate/core mass fractions in mol% (or fractional) in tables and plots
 FigLbl.qSURF_IN_mW = True  # Whether to print qSurf in mW/m^2 (or W/m^2)
 FigLbl.phi_IN_VOLPCT = False  # Whether to print porosity (phi) in vol% (or unitless volume fraction)
 FigLbl.SetUnits()  # Make use of above toggles and assign labels
 
 
 """ Figure sizes """
-FigSize.vsP = (3, 3)
-FigSize.vsR = (3, 3)
-FigSize.vperm = (3, 3)
-FigSize.vgsks = (3, 3)
-FigSize.vseis = (3, 3)
-FigSize.vhydro = (8, 5)
+FigSize.vpore = (6, 6)
+FigSize.vperm = (6, 6)
+FigSize.vseis = (6, 6)
+FigSize.vhydro = (9, 5)
 FigSize.vgrav = (6, 5)
 FigSize.vmant = (6, 6)
 FigSize.vcore = (6, 6)
-FigSize.vpvt4 = (3, 3)
-FigSize.vpvt6 = (3, 3)
+FigSize.vpvt = (12, 6)
 FigSize.vwedg = (4.5, 4.5)
+FigSize.explore = (6, 4)
 FigSize.phaseSpaceSolo = (6, 4)
 FigSize.phaseSpaceCombo = (9, 4)
 FigSize.induct = (8, 4)
 FigSize.inductCombo = (8, 8)
+FigSize.Bdip = (5, 3)
+FigSize.BdipCombo = (6, 9)
+FigSize.BdipSolo = (2.5, 3)
+FigSize.BdipSoloCombo = (3, 9)
 
 
 """ Miscellaneous figure options """
@@ -158,6 +178,12 @@ FigMisc.defaultFontName = 'STIXGeneral'  # Default font variables--STIX is what
 FigMisc.defaultFontCode = 'stix'  # Code name for default font needed in some function calls
 FigMisc.backupFont = 'Times New Roman'  # Backup font that looks similar to STIX that most users are likely to have
 
+# Hydrosphere plots
+FigMisc.SCALE_HYDRO_LW = True  # Whether to adjust thickness of lines on hydrosphere plot according to relative salinity
+FigMisc.MANUAL_HYDRO_COLORS = True  # Whether to set color of lines in hydrosphere according to melting temperature
+FigMisc.RELATIVE_Tb_K = True  # Whether to set colormap of lines based on relative comparison (or fixed settings in ColorStruct)
+FigMisc.TminHydro = 200  # Minimum temperature to display on hydrosphere plots
+
 # Wedge diagrams
 FigMisc.IONOSPHERE_IN_WEDGE = False  # Whether to include specified ionosphere in wedge diagram
 FigMisc.WEDGE_ICE_TICKS = False  # Whether to print ticks for ice shell, which usually overlap with the body outer radius
@@ -166,6 +192,16 @@ FigMisc.DRAW_CONVECTION_BOUND = False  # Whether to draw a boundary line between
 FigMisc.DRAW_POROUS_BOUND = False  # Whether to draw a boundary line between porous and non-porous materials
 FigMisc.DRAW_FeS_BOUND = True  # Whether to draw a boundary line between Fe and FeS in the core
 
+# Silicate/core PT diagrams
+FigMisc.nTgeo = 80  # Number of temperature points to evaluate/plot for PT property plots
+FigMisc.nPgeo = 100  # Number of pressure points to evaluate/plot for PT property plots
+FigMisc.nPgeoCore = 40  # Subset of nPgeo to use for core, if present
+FigMisc.PVT_INCLUDE_CORE = False  # Whether to include core as well as silicates in PT properties diagrams
+        
+# Induced dipole surface strength plots
+FigMisc.BdipZoomMult = 1.05  # Extra space to include around zoomed-in part, in fraction of largest value.
+FigMisc.SHOW_INSET = True  # Whether to show the inset box for the zoom-in plot, when applicable
+
 # Inductogram phase space plots
 FigMisc.DARKEN_SALINITIES = False  # Whether to match hues to the colorbar, but darken points based on salinity, or to just use the colorbar colors.
 FigMisc.NORMALIZED_SALINITIES = False  # Whether to normalize salinities to absolute concentrations relative to the saturation limit for each salt
@@ -177,11 +213,9 @@ FigMisc.PLOT_V2021 = True  # Mark the selected ocean/conductivity combos used in
 FigMisc.DO_PER = True  # Convert frequency axes to periods for FFT plots
 
 # Legends
-FigMisc.LEGEND = True  # Whether to plot legends
 FigMisc.REFS_IN_LEGEND = True  # Hydrosphere plot: Whether to include reference profiles in legend
-FigMisc.hydroLegendBox = (0.15, 0.1, 0.33, 0.5)  # Hydrosphere plot: Bounding box for where to place legends. Values are x, y, dx, dy in fractions of the figure size, where x and y are for the bottom-left corner of the box.
-FigMisc.hydroLegendPos = 'center left'  # Hydrosphere plot: Where to place legends within bounding box
 FigMisc.wedgeLegendPos = 'center right'  # Wedge diagram: Where in axes added at right to place legend
+FigMisc.legendFontSize = 'x-small'  # Font size to use in legends, set by rcParams.
 
 # Latex settings
 FigMisc.fontFamily = 'serif'
@@ -206,6 +240,7 @@ FigMisc.cLabelPad = 5  # Padding in pt to set beside contour labels
 FigMisc.cLegendOpacity = 1.0  # Opacity of legend backgrounds in contour plots.
 
 # Colorbar settings
+FigMisc.cbarTitleSize = 'small'  # Font size specifier for colorbar titles
 FigMisc.cbarSpace = 0.5  # Amount of whitespace in inches to use for colorbars
 FigMisc.cbarSize = '5%'  # Description of the size of colorbar to use with make_axes_locatable
 FigMisc.cbarHeight = 0.6  # Fraction of total figure height to use for colorbar size
@@ -214,3 +249,4 @@ FigMisc.extraPad = FigMisc.cbarSpace * 0.8  # Amount of extra padding to apply t
 FigMisc.cbarFmt = '%.1f'  # Format string to use for colorbar units
 FigMisc.nCbarPts = 80  # Number of points to use for drawing colorbar gradient
 FigMisc.cbarBottom = (1 - FigMisc.cbarHeight - FigMisc.cbarPad*2/72)/2  # Fraction of total figure height to use for bottom edge of colorbar
+FigMisc.SetFontSizes()  # Assign sizes for fonts
